package raytracer.geometricObjects;
import raytracer.utilities.RGBColor;
import raytracer.utilities.Ray;
import raytracer.utilities.ShadeRec;

//Copyright (C) Helen Hu 2011.
//based on C++ code from Ray Tracing from the Ground Up, by Kevin Suffern 
//This Java code is for non-commercial purposes only.
//This Java code is licensed under the GNU General Public License Version 2.
//See the file COPYING.txt for the full license.

/** this file contains the definition of the class GeometricObject 
 *
 */

public abstract class GeometricObject {
	
	protected RGBColor   color;						// only used for Bare Bones ray tracing

	// ---------------------------------------------------------------------- default constructor

	public GeometricObject() {
		color = new RGBColor(RGBColor.black);
	}
	
	// ---------------------------------------------------------------------- copy constructor

	public GeometricObject (GeometricObject object) {
		this.color = new RGBColor(object.color);	
	}


	// ---------------------------------------------------------------------- assignment operator

	public void set (GeometricObject rhs) {
		
		if (this != rhs) 
			this.color.set(rhs.color);
	}

	// ---------------------------------------------------------------------- abstract methods
	
	// virtual copy constructor
	public abstract GeometricObject clone();
	
	public abstract double hit(Ray ray, ShadeRec s);
	
	// the following three functions are only required for Chapter 3
	

	// --------------------------------------------------------------------  set_colour

	public void set_color(RGBColor c) {
		color.set(c);
	}

	// --------------------------------------------------------------------  set_colour

	public void set_color(float r, float g, float b) {
		color.r = r;
		color.b = b;
		color.g = g;
	}

	// --------------------------------------------------------------------  get_colour

	public RGBColor get_color() {
		return (color);
	}

	public boolean equals(Object obj) {
		if (obj == null || this.getClass() != obj.getClass()) {
			return false;
		}
		else {
			GeometricObject other = (GeometricObject)obj;
			return (color.equals(other.color));
		}
	}
	
	public String toString() {
		return "Geometry Object with color " + color;
	}
}
