package raytracer.utilities;
import raytracer.world.World;

//Copyright (C) Helen Hu 2011.
//based on C++ code from Ray Tracing from the Ground Up, by Kevin Suffern 
//This Java code is for non-commercial purposes only.
//This Java code is licensed under the GNU General Public License Version 2.
//See the file COPYING.txt for the full license.


public class ShadeRec {
	public boolean 			hit_an_object;		// did the ray hit an object?
	public Point3D			local_hit_point;	// world coordinates of hit point 
	public Normal			normal;				// normal at hit point
	public RGBColor			color;				// used in Chapter 3 only
	public World			w;					// world reference for shading

	// ------------------------------------------------------------------ constructor

	public ShadeRec(World wr) { 
		hit_an_object = false;
		local_hit_point = new Point3D();
		normal = new Normal();
		color = new RGBColor(RGBColor.black);
		w = wr;
	}


	// ------------------------------------------------------------------ copy constructor

	public ShadeRec(ShadeRec sr) {
		hit_an_object = sr.hit_an_object;
		local_hit_point = new Point3D(sr.local_hit_point);
		normal = new Normal();
		color = new RGBColor(sr.color);
		w = sr.w;	
	}
	
	public boolean equals(Object obj) {
		if (obj == null || this.getClass() != obj.getClass()) {
			return false;
		}
		else {
			ShadeRec other = (ShadeRec)obj;
			return (this.hit_an_object==other.hit_an_object && 
					this.equals(other.local_hit_point) && 
					this.normal.equals(other.normal) &&
					this.color.equals(other.color)  &&
					this.w == other.w);
		}
	}
	
	public String toString() {
		if (hit_an_object)	
			return "ShadeRec: hit " + local_hit_point + " at " + normal;
		else
			return "ShadeRec: nothing hit";
	}
}
