package raytracer.world;

//Copyright (C) Helen Hu 2011.
//based on C++ code from Ray Tracing from the Ground Up, by Kevin Suffern 
//This Java code is for non-commercial purposes only.
//This Java code is licensed under the GNU General Public License Version 2.
//See the file COPYING.txt for the full license.

public class ViewPlane {
	public int 			hres;   					// horizontal image resolution 
	public int 			vres;   					// vertical image resolution
	public float		s;							// pixel size
		
	public float		gamma;						// gamma correction factor
	public float		inv_gamma;					// the inverse of the gamma correction factor
	public boolean		show_out_of_gamut;			// display red if RGBColor out of gamut
	

	// ---------------------------------------------------------------- default constructor	
							
	public ViewPlane() {							
		hres = 400;
		vres = 400;
		s = 1f;
		gamma = 1f;
		inv_gamma = 1f;
		show_out_of_gamut = false;
	}

	// ---------------------------------------------------------------- copy constructor

	public ViewPlane(ViewPlane vp){
		hres = vp.hres;
		vres = vp.vres;
		s = vp.s;
		gamma = vp.gamma;
		inv_gamma = vp.inv_gamma;
		show_out_of_gamut = vp.show_out_of_gamut;
	}

	// ---------------------------------------------------------------- assignment operator

	public void set (ViewPlane rhs) {
		if (this != rhs) {			
			hres 				= rhs.hres;
			vres 				= rhs.vres;
			s					= rhs.s;
			gamma				= rhs.gamma;
			inv_gamma			= rhs.inv_gamma;
			show_out_of_gamut	= rhs.show_out_of_gamut;
		}
	}

	// ------------------------------------------------------------------------------ set_hres

	public void set_hres(int h_res) {
		hres = h_res;
	}

	// ------------------------------------------------------------------------------ set_vres

	public void set_vres(int v_res) {
		vres = v_res;
	}

	// ------------------------------------------------------------------------------ set_pixel_size

	public void set_pixel_size(float size) {
		s = size;
	}


	// ------------------------------------------------------------------------------ set_gamma

	public void set_gamma(float g) {
		gamma = g;
		inv_gamma = 1.0f / gamma;
	}

	// ------------------------------------------------------------------------------ set_gamut_display

	public void set_gamut_display(boolean show) {
		show_out_of_gamut = show;
	}
				
	public boolean equals(Object obj) {
		if (obj == null || this.getClass() != obj.getClass()) {
			return false;
		}
		else {
			ViewPlane vp = (ViewPlane)obj;
			return (hres == vp.hres && 
					vres == vp.vres &&
					s == vp.s &&
					gamma == vp.gamma &&
					inv_gamma == vp.inv_gamma &&
					show_out_of_gamut == vp.show_out_of_gamut);
		}
	}

	public String toString() {
		return "View Plane: " + hres + " x " + vres + ", pixel size" + s + "(gamma = " + gamma + ")";
	}
}
